/***
*
* BNBT Beta 8.0 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 Trevor Hogan
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

// =Xotic= Modified Source File

#include "bnbt.h"
#include "atom.h"
#include "tracker.h"
#include "server.h"
#include "util.h"

// Added by =Xotic=
#include "link.h"  
#include "xconfig_lang.h"

void CTracker :: serverResponseAdmin( struct request_t *pRequest, struct response_t *pResponse )
{
	// Modified by =Xotic=
	struct bnbttv btv = UTIL_CurrentTime( );
	
	if( pRequest->user.iAccess & ACCESS_ADMIN )
	{
		//
		// kill tracker
		//

		if( pRequest->mapParams["ap_kill"] == "1" )
		{
			gpServer->Kill( );

			return;
		}

		if( m_bCountUniquePeers && pRequest->mapParams["ap_recount"] == "1" )
		{
			gpServer->getTracker( )->CountUniquePeers( );

			// Modified by =Xotic=
			JS_ReturnToAdmin( pResponse );
			// -------------------------- End of Modification

			return;
		}

		//
		// reset tracker link
		//
		if( pRequest->mapParams["ap_relink"] == "1" )
		{
			if( gpLink )
			{
				// Resetting tracker link
				gpLink->Kill( );
	
				JS_ReturnToAdmin( pResponse );
			}
			else
			{
				// This tracker does not own a tracker link (it might be a tracker hub)
				m_bFlagNotOwnLinkAlert = true;
				JS_ReturnToAdmin( pResponse );
			}

			return;
		}  
			
		// Modified by =Xotic=
		// Dump XML

		if( !xmldump.strName.empty( ) && pRequest->mapParams["ap_xml"] == "1" )
		{
			if( gbDebug )
				UTIL_LogPrint( "tracker - dumping XML\n" );

			saveXML( );

			m_bFlagXMLAlert = true;
			JS_ReturnToAdmin( pResponse );
			return;
		}

		// addition by labarks

		if( !rssdump.strName.empty( ) && pRequest->mapParams["ap_rss"] == "1" )
		{
			if( m_iDumpRSSFileMode == 0 || m_iDumpRSSFileMode == 2 )
			{
				if( gbDebug )
					UTIL_LogPrint( "tracker - dumping RSS\n" );

				saveRSS( );
			}

			if( m_iDumpRSSFileMode == 1 || m_iDumpRSSFileMode == 2 )
			{
				for( vector< pair<string, string> > :: iterator i = m_vecTags.begin( ); i != m_vecTags.end( ); i++ )
				{
					string strTag = (string)(*i).first;

					if( gbDebug )
						UTIL_LogPrint( "tracker - dumping RSS for %s\n", strTag.c_str( ) );

					saveRSS( strTag );
				}

				if( !m_vecTags.size( ) && m_iDumpRSSFileMode == 1 )
					UTIL_LogPrint( "tracker warning - no tags to dump RSS files per category, try changing to mode 0 or 2\n" );
			}
			m_bFlagRSSAlert = true;
			JS_ReturnToAdmin( pResponse );
			return;
		}
		// end addition

		if( pRequest->mapParams["ap_dynstat"] == "1" )
		{
			resetXStatsDynstat( );
			saveXStats( );
			JS_ReturnToAdmin( pResponse );
			return;
		}

		if( pRequest->mapParams["ap_announce"] == "1" )
		{
			resetXStatsAnnounce( );
			saveXStats( );
			JS_ReturnToAdmin( pResponse );
			return;
		}

		if( pRequest->mapParams["ap_scrape"] == "1" )
		{
			resetXStatsScrape( );
			saveXStats( );
			JS_ReturnToAdmin( pResponse );
			return;
		}

		if( pRequest->mapParams["ap_file"] == "1" )
		{
			resetXStatsFile( );
			saveXStats( );
			JS_ReturnToAdmin( pResponse );
			return;
		}

		if( pRequest->mapParams["ap_page"] == "1" )
		{
			resetXStatsPage( );
			saveXStats( );
			JS_ReturnToAdmin( pResponse );
			return;
		}

		if( pRequest->mapParams["ap_all"] == "1" )
		{
			resetXStatsAll( );
			saveXStats( );
			JS_ReturnToAdmin( pResponse );
			return;
		}

		// XBNBT Stats
		HTML_Common_Begin( pRequest, pResponse, gmapLANG_CFG["admin_page"], string( CSS_ADMIN ), string( ), NOT_INDEX, CODE_200 );
		JS_Admin( pResponse );
		
		// Tables
#ifdef XBNBT_GD
		tableXStatsDynstat( pRequest, pResponse );
#endif
		tableXStatsAnnounce( pRequest, pResponse );
		tableXStatsScrape( pRequest, pResponse );
		tableXStatsFile( pRequest, pResponse );
		tableXStatsPage( pRequest, pResponse );
		// -------------------------- End of Modification

		//
		// tracker links
		//
		pResponse->strContent += "<div class=\"admin_table\">\n";
		pResponse->strContent += "<table summary=\"tlink\">\n";
		
		if( gpLink )
		{
			pResponse->strContent += "<tr><th colspan=1>" + gmapLANG_CFG["admin_tracker_links"] + "</th>\n";
			pResponse->strContent += "<th colspan=1>";
			Button_JS_Confirm( "confirm_relink", gmapLANG_CFG["reset"], "relink_confirm()", pResponse );
			pResponse->strContent += "</th></tr>\n";

			pResponse->strContent += "<tr><th>Established</th>";
			pResponse->strContent += "<td class=\"tracker_connection\">" + gstrLinkEstDate + "</td></tr>\n";
		}
		else
		{
			pResponse->strContent += "<tr><th colspan=2>" + gmapLANG_CFG["admin_tracker_links"] + "</th></tr>\n";
		}

		pResponse->strContent += "<tr><th>Type</th>";

		if( gpLinkServer )
		{
			pResponse->strContent += "<td class=\"tracker_link_type\">" + gmapLANG_CFG["admin_primary_tracker"] + "</td></tr>\n";
			pResponse->strContent += "<tr><th>" + gmapLANG_CFG["admin_connections"] + "</th><td class=\"tracker_connection\">";

			gpLinkServer->m_mtxLinks.Claim( );

			for( vector<CLinkClient *> :: iterator i = gpLinkServer->m_vecLinks.begin( ); i != gpLinkServer->m_vecLinks.end( ); i++ )
			{
				pResponse->strContent += (*i)->getName( );

				if( (*i)->m_bActive )
					pResponse->strContent += " " + gmapLANG_CFG["admin_active"];
				else
					pResponse->strContent += " " + gmapLANG_CFG["admin_not_active"];

				if( i + 1 != gpLinkServer->m_vecLinks.end( ) )
					pResponse->strContent += "<br>";
			}

			gpLinkServer->m_mtxLinks.Release( );

			pResponse->strContent += "</td></tr>\n";
		}
		else if( gpLink )
		{
			pResponse->strContent += "<td class=\"tracker_link_type\">" + gmapLANG_CFG["admin_secondary_tracker"] + "</td></tr>\n";
			pResponse->strContent += "<tr><th>" + gmapLANG_CFG["admin_connections"] + "</th><td class=\"tracker_connection\">" + gpLink->getName( ) + "</td></tr>\n";
		}
		else
			pResponse->strContent += "<td class=\"tracker_connection\">" + gmapLANG_CFG["admin_no_link"] + "</td></tr>\n";

		pResponse->strContent += "</table>\n";
		pResponse->strContent += "</div>\n";

		//
		// clients
		//
		int iOnline = ( int )gpServer->m_vecClients.size( );

		pResponse->strContent += "<p class=\"users_online\">" + gmapLANG_CFG["admin_serving"] + " ";
		pResponse->strContent += CAtomInt( iOnline ).toString( ) + " ";

		if ( iOnline > 1 )
			pResponse->strContent += gmapLANG_CFG["admin_serving_clients"];
		else
			pResponse->strContent += gmapLANG_CFG["admin_serving_client"];

		pResponse->strContent += "</p>\n";

		// Button List
		pResponse->strContent += "<p class=\"button_list\">";

		// Reset all stats
		Button_JS_Confirm( "confirm_all", gmapLANG_CFG["reset_all_stats"], "all_confirm()", pResponse );

		// count unique peers
		if( m_bCountUniquePeers )
			Button_JS_Link( "link_count", gmapLANG_CFG["admin_count_peers"] + " (" + CAtomLong( m_pIPs->getValuePtr( )->size( ) ).toString( ) + ")", "admin_count()", pResponse );

		// RSS
		// addition by labarks
		if( !rssdump.strName.empty( ) )
			Button_JS_Link( "link_rss", gmapLANG_CFG["admin_update_rss"], "admin_rss()", pResponse );

		// XML
		if( !rssdump.strName.empty( ) )
			Button_JS_Link( "link_xml", gmapLANG_CFG["admin_update_xml"], "admin_xml()", pResponse );

		pResponse->strContent += "</p>\n";

		//
		// kill tracker
		//  
		pResponse->strContent += "<p class=\"tracker_kill\">";
		Button_JS_Confirm( "confirm_kill", gmapLANG_CFG["admin_kill_tracker"], "kill_confirm()", pResponse );
		pResponse->strContent += "</p>\n";
		pResponse->strContent += "<p class=\"tracker_kill_warning\">" + gmapLANG_CFG["admin_kill_warning"] + "</p>\n";

		// Output common HTML ending
		HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_ADMIN ) );
	}
	else
	{
		HTML_Common_Begin(  pRequest, pResponse, gmapLANG_CFG["admin_page"], string( CSS_ADMIN ), string( ), NOT_INDEX, CODE_401 );

		// Output common HTML ending
		HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_ADMIN ) );
	}
}
