/***
*
* BNBT Beta 8.0 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 Trevor Hogan
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

// =Xotic= Modified Source File

#include "bnbt.h"
#include "atom.h"
#include "tracker.h"
#include "util.h"

// Added by =Xotic=
#include "sort.h"
#include "xconfig_lang.h"
// --------------------- End of Addition

void CTracker :: serverResponseLogin( struct request_t *pRequest, struct response_t *pResponse )
{
	struct bnbttv btv = UTIL_CurrentTime( );

	// Not authorized
	if( pRequest->user.strLogin.empty( ) )
	{
		// Output common HTML begin
		HTML_Common_Begin(  pRequest, pResponse, gmapLANG_CFG["login_page"], string( CSS_LOGIN ), string( ), NOT_INDEX, CODE_401 );

		// Output common HTML end
		HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_LOGIN ) );
		// --------------------------------- End of Modification

		return;
	}

	// cookies

	time_t tNow = time( NULL );

	struct tm tmFuture = *gmtime( &tNow );
	tmFuture.tm_mon++;
	mktime( &tmFuture );
	struct tm tmPast = *gmtime( &tNow );
	tmPast.tm_mon--;
	mktime( &tmPast );

	char pTime[256];
	memset( pTime, 0, sizeof( char ) * 256 );

	string strLogout = pRequest->mapParams["logout"];

	if( strLogout == "1" )
		strftime( pTime, sizeof( char ) * 256, "%a, %d-%b-%Y %H:%M:%S GMT", &tmPast );
	else
		strftime( pTime, sizeof( char ) * 256, "%a, %d-%b-%Y %H:%M:%S GMT", &tmFuture );

	pResponse->mapHeaders.insert( pair<string, string>( "Pragma", "No-Cache" ) );
	pResponse->mapHeaders.insert( pair<string, string>( "Set-Cookie", "login=\"" + pRequest->user.strLogin + "\"; expires=" + pTime + "; path=/" ) );
	pResponse->mapHeaders.insert( pair<string, string>( "Set-Cookie", "md5=\"" + UTIL_StringToEscaped( pRequest->user.strMD5 ) + "\"; expires=" + pTime + "; path=/" ) );

	// Modified by =Xotic=
	HTML_Common_Begin( pRequest, pResponse, gmapLANG_CFG["login_page"], string( CSS_LOGIN ), string( ), NOT_INDEX, CODE_200 );
	// --------------------- End of Modification

	if( strLogout == "1" ) 
		pResponse->strContent += "<p class=\"logout\">" + gmapLANG_CFG["logging_out"] + "</p>\n";
	else
	{
		if( m_bDeleteOwnTorrents )
		{
			if( pRequest->mapParams.find( "del" ) != pRequest->mapParams.end( ) )
			{
				string strDelHashString = pRequest->mapParams["del"];
				string strDelHash = UTIL_StringToHash( strDelHashString );
				string strOK = pRequest->mapParams["ok"];

				if( strDelHash.empty( ) )
				{
					// Modified by =Xotic=	
					// Unable to delete torrent. The info hash is invalid.
					pResponse->strContent += "<p class=\"delete\">" + UTIL_Xsprintf( gmapLANG_CFG["login_invalid_hash"].c_str( ), strDelHashString.c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_login"] + "\" href=\"/login.html\">" ).c_str( ), "</a>" ) + "</p>\n";

					// Output common HTML ending
					HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_LOGIN ) );
					// ---------------------- End of Modification

					return;
				}
				else
				{
					if( strOK == "1" )
					{
						if( m_pTags )
						{
							CAtom *pTagInfo = m_pTags->getItem( strDelHash );

							if( pTagInfo && pTagInfo->isDicti( ) )
							{
								CAtom *pUploader = ( (CAtomDicti *)pTagInfo )->getItem( "uploader" );

								string strUploader;

								if( pUploader )
									strUploader = pUploader->toString( );

								if( strUploader != pRequest->user.strLogin )
								{
									// Modified by =Xotic=	
									// Unable to delete torrent. You didn't upload that torrent.
									pResponse->strContent += "<p class=\"delete\">" + UTIL_Xsprintf( gmapLANG_CFG["login_not_owned"].c_str( ), strDelHashString.c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_login"] + " href=\"/login.html\">" ).c_str( ), "</a>" ) + "</p>\n";

									// Output common HTML ending
									HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_LOGIN ) );
									// ---------------------- End of Modificationo

									return;
								}
							}
						}

						if( m_pAllowed )
						{
							// delete from disk

							CAtom *pList = m_pAllowed->getItem( strDelHash );

							if( pList && dynamic_cast<CAtomList *>( pList ) )
							{
								vector<CAtom *> vecTorrent = dynamic_cast<CAtomList *>( pList )->getValue( );

								if( vecTorrent.size( ) == 6 )
								{
									CAtom *pFileName = vecTorrent[0];

									if( pFileName )
									{
										if( m_strArchiveDir.empty( ) )
											UTIL_DeleteFile( ( m_strAllowedDir + pFileName->toString( ) ).c_str( ) );
										else
											UTIL_MoveFile( ( m_strAllowedDir + pFileName->toString( ) ).c_str( ), ( m_strArchiveDir + pFileName->toString( ) ).c_str( ) );
									}
								}
							}

							m_pAllowed->delItem( strDelHash );
							m_pDFile->delItem( strDelHash );
							deleteTag( strDelHash );
									
							// Modified by =Xotic=	
							// Deleted torrent
							pResponse->strContent += "<p class=\"deleted\">" + UTIL_Xsprintf( gmapLANG_CFG["login_deleted_torrent"].c_str( ), strDelHashString.c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_login"] + " href=\"/login.html\">" ).c_str( ), "</a>" ) + "</p>\n";

							// Output common HTML ending
							HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_LOGIN ) );
							// ---------------------- End of Modification

							return;
						}
					}
					else
					{
						// Modified by =Xotic=
						// Are you sure you want to delete the torrent
						pResponse->strContent += "<p class=\"delete\">" + UTIL_Xsprintf( gmapLANG_CFG["delete_torrent_q"].c_str( ), strDelHashString.c_str( ) ) + "</p>\n";
						pResponse->strContent += "<p class=\"delete\"><a title=\"" + gmapLANG_CFG["yes"] + " href=\"/index.html?del=" + strDelHashString + "&amp;ok=1\">" + gmapLANG_CFG["yes"] + "</a>\n";
						pResponse->strContent += " | <a title=\"" + gmapLANG_CFG["no"] + " href=\"" + string( JS_BACK ) + "\">" + gmapLANG_CFG["no"] + "</a></p>\n";

						// Output common HTML ending
						HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_LOGIN ) );
						// ---------------------- End of Modification

						return;
					}
				}
			}
		}

		// Modified by =Xotic=
		if( m_pTags )
		{
			map<string, CAtom *> *pmapDicti = m_pTags->getValuePtr( );

			unsigned long iKeySize = ( unsigned long )pmapDicti->size( );

			// add the torrents into this structure one by one and sort it afterwards

			struct torrent_t *pTorrents = new struct torrent_t[iKeySize];

			unsigned long i = 0;

			// Load into pTorrents, indexed by InfoHash
			for( map<string, CAtom *> :: iterator it = pmapDicti->begin( ); it != pmapDicti->end( ); it++ )
			{
				pTorrents[i].strInfoHash = (*it).first;
				pTorrents[i].strName = gmapLANG_CFG["unknown"];
				pTorrents[i].strLowerName = gmapLANG_CFG["unknown"];
				pTorrents[i].strTag = gmapLANG_CFG["unknown"];
				pTorrents[i].strUploader = gmapLANG_CFG["unknown"];
				pTorrents[i].strAdded = gmapLANG_CFG["unknown"];

				if( (*it).second->isDicti( ) )
				{
					// Tag
					CAtom *pTag = ( (CAtomDicti *)(*it).second )->getItem( "tag" );

					if( pTag )
						pTorrents[i].strTag = pTag->toString( );

					// Uploader
					CAtom *pUploader = ( (CAtomDicti *)(*it).second )->getItem( "uploader" );

					if( pUploader )
						pTorrents[i].strUploader = pUploader->toString( );

					// Name of torrent
					CAtom *pName = ( (CAtomDicti *)(*it).second )->getItem( "name" );

					if( pName )
					{
						pTorrents[i].strName = pName->toString( );
						pTorrents[i].strLowerName = UTIL_ToLower( pTorrents[i].strName );
					}
				}

				// Added date
				CAtom *pList = m_pAllowed->getItem( pTorrents[i].strInfoHash );

				if( pList && dynamic_cast<CAtomList *>( pList ) )
				{
					vector<CAtom *> vecTorrent = ( (CAtomList *)pList )->getValue( );

					if( vecTorrent.size( ) == 6 )
					{
						CAtom *pAdded = vecTorrent[2];

						if( pAdded )
							pTorrents[i].strAdded = pAdded->toString( );

					}
				}

				i++;
			}

			// Sort
			string strSort = pRequest->mapParams["sort"];

			if( m_bSort )
			{
				if( !strSort.empty( ) )
				{
					int iSort = atoi( strSort.c_str( ) );

					// Modified by =Xotic=
					switch( iSort )
					{
					case SORT_ANAME:
						if( m_pAllowed )
							qsort( pTorrents, iKeySize, sizeof( struct torrent_t ), asortByName );
						break;
					case SORT_DNAME:
						if( m_pAllowed )
							qsort( pTorrents, iKeySize, sizeof( struct torrent_t ), dsortByName );
						break;
					case SORT_AADDED:
						if( m_pAllowed )
							qsort( pTorrents, iKeySize, sizeof( struct torrent_t ), asortByAdded );
						break;
					case SORT_ATAG:
						qsort( pTorrents, iKeySize, sizeof( struct torrent_t ), asortByTag );
						break;
					case SORT_DTAG:
						qsort( pTorrents, iKeySize, sizeof( struct torrent_t ), dsortByTag );
						break;
					case SORT_DADDED:
					default:
						// default action is to sort by dadded if we can
						if( m_pAllowed )
							qsort( pTorrents, iKeySize, sizeof( struct torrent_t ), dsortByAdded );
					}
					// --------------------------- End of Modification
				}
				else
				{
					// default action is to sort by dadded if we can
					if( m_pAllowed )
						qsort( pTorrents, iKeySize, sizeof( struct torrent_t ), dsortByAdded );
				}
			}
			else
			{
				// sort is disabled, but default action is to sort by dadded if we can
				if( m_pAllowed )
					qsort( pTorrents, iKeySize, sizeof( struct torrent_t ), dsortByAdded );
			}

			// some preliminary search crap
			string strSearch = pRequest->mapParams["search"];
			string strLowerSearch = UTIL_ToLower( strSearch );
			string strSearchResp = UTIL_StringToEscaped( strSearch );

			// Count matching torrents for top of page
			int iFound = 0;

			for( unsigned long i = 0; i < iKeySize; i++ )
			{
				if( !strSearch.empty( ) )
				{
					// only count entries that match the search
					if( pTorrents[i].strLowerName.find( strLowerSearch ) == string :: npos )
						continue;
				}

				// only count entries that match the current user
				if( pTorrents[i].strUploader != pRequest->user.strLogin )
					continue;

				iFound++;
			}

			// Modified by =Xotic=
			// You signed up on
			pResponse->strContent += "<p class=\"registered\">" + UTIL_Xsprintf( gmapLANG_CFG["login_signed_up"].c_str( ), pRequest->user.strCreated.c_str( ) ) + "</p>\n\n";
			// ----------------------------------------- End of Modification

			// search messages
			if( m_pAllowed && m_bSearch )
			{
				pResponse->strContent += "<p class=\"search_filter\">\n";

				if( !strSearch.empty() )
					pResponse->strContent += "<span class=\"search_results_alt\">Search results for: \"</span><span class=\"filtered_by\">" + UTIL_RemoveHTML( strSearch ) + "\"</span>\n";

				pResponse->strContent += "</p>\n\n";

				if( iFound == 0 )
					pResponse->strContent += "<p class=\"results\">" + gmapLANG_CFG["result_none_found"] + "</p>\n\n";
				else if( iFound == 1 )
					pResponse->strContent += "<p class=\"results\">" + gmapLANG_CFG["result_1_found"] + "</p>\n\n";
				else
					// Modified by =Xotic=
					// Results found
					pResponse->strContent += "<p class=\"results\">" + UTIL_Xsprintf( gmapLANG_CFG["result_x_found"].c_str( ), CAtomInt( iFound ).toString( ).c_str( ) ) + "</p>\n\n";
					// ---------------------------- End of Modification
			}

			// Top search
			if( ( !strSearch.empty( ) || iFound ) && m_pAllowed && m_bSearch )
			{
				pResponse->strContent += "<form class=\"search_login_top\" name=\"topsearch\" method=\"get\" action=\"/login.html\">\n";

				if( !strSort.empty( ) )
					pResponse->strContent += "<p><input name=\"sort\" type=hidden value=\"" + strSort + "\"></p>\n";

				pResponse->strContent += "<p><label for=\"toptorrentsearch\">" + gmapLANG_CFG["torrent_search"] + "</label> <input name=\"search\" id=\"toptorrentsearch\" alt=\"[" + gmapLANG_CFG["torrent_search"] + "]\" type=text size=40> <a title=\"" + gmapLANG_CFG["clear_filter_search"] + "\" href=\"/index.html\">" + gmapLANG_CFG["clear_filter_search"] + "</a></p>\n";

				pResponse->strContent += "</form>\n\n";
			}

			int iAdded = 0;
			bool bFound = false;

			for( unsigned long j = 0; j < iKeySize; j++ )
			{
				// only display entries that match the current user
				if( pTorrents[j].strUploader != pRequest->user.strLogin )
					continue;

				// only display entries that match the search
				if( !strSearch.empty( ) )
				{
					if( pTorrents[j].strLowerName.find( strLowerSearch ) == string :: npos )
						continue;
				}

				// create the table and display the headers first
				if( !bFound )
				{
					pResponse->strContent += "<div class=\"login_table\">\n";
					pResponse->strContent += "<table summary=\"ytinfo\">\n";
					pResponse->strContent += "<tr><th colspan=4>" + gmapLANG_CFG["login_your_torrents"] + "</th></tr>\n";

					pResponse->strContent += "<tr>\n";

					// <th> tag
					if( !m_vecTags.empty( ) )
					{
						pResponse->strContent += "<th class=\"tag\">" + gmapLANG_CFG["tag"];
						if( m_bSort )
						{
							pResponse->strContent += "<br>\n<div><a class=\"sort\" title=\"" + gmapLANG_CFG["sort_tag_ascending"] + "\" href=\"/login.html?sort=" + SORTSTR_ATAG;

							if( !strSearch.empty( ) )
								pResponse->strContent += "&amp;search=" + strSearchResp;

							pResponse->strContent += "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a class=\"sort\" title=\"" + gmapLANG_CFG["sort_tag_descending"] + "\" href=\"/login.html?sort=" + SORTSTR_DTAG;

							if( !strSearch.empty( ) )
								pResponse->strContent += "&amp;search=" + strSearchResp;

							pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a></div>";
						}

						pResponse->strContent += "</th>\n";
					}

					// Name
					pResponse->strContent += "<th class=\"name\">" + gmapLANG_CFG["name"];

					if( m_bSort )
					{
						pResponse->strContent += "<br>\n<div><a class=\"sort\" title=\"" + gmapLANG_CFG["sort_name_ascending"] + "\" href=\"/login.html?sort=" + SORTSTR_ANAME;

						if( !strSearch.empty( ) )
							pResponse->strContent += "&amp;search=" + strSearchResp;

						pResponse->strContent +=  + "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a class=\"sort\" title=\"" + gmapLANG_CFG["sort_name_descending"] + "\" href=\"/login.html?sort=" + SORTSTR_DNAME;

						if( !strSearch.empty( ) )
							pResponse->strContent += "&amp;search=" + strSearchResp;

						pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a></div>";
					}

					pResponse->strContent += "</th>\n";						

					// Added
					pResponse->strContent += "<th>" + gmapLANG_CFG["added"];

					if( m_bSort )
					{
						pResponse->strContent += "<br>\n<div><a class=\"sort\" title=\"" + gmapLANG_CFG["sort_added_ascending"] + "\" href=\"/login.html?sort=" + SORTSTR_AADDED;

						if( !strSearch.empty( ) )
							pResponse->strContent += "&amp;search=" + strSearchResp;

						pResponse->strContent +=  + "\">" + gmapLANG_CFG["sort_ascending"] + "</a><span>-</span><a class=\"sort\" title=\"" + gmapLANG_CFG["sort_added_descending"] + "\" href=\"/login.html?sort=" + SORTSTR_DADDED;

						if( !strSearch.empty( ) )
							pResponse->strContent += "&amp;search=" + strSearchResp;

						pResponse->strContent += "\">" + gmapLANG_CFG["sort_descending"] + "</a></div>";
					}

					pResponse->strContent += "</th>\n";

					// Admin
					pResponse->strContent += "<th>" + gmapLANG_CFG["admin"] + "</th>\n";

					// end row
					pResponse->strContent += "</tr>\n";

					// signal table created
					bFound = true;
				}

				// set the row colour
				if( iAdded % 2 )
					pResponse->strContent += "<tr class=\"even\">\n";
				else
					pResponse->strContent += "<tr class=\"odd\">\n";

				// display the tag
				pResponse->strContent += "<td>" ;
				if( !m_vecTags.empty( ) )
				{
					for( vector< pair< string, string > > :: iterator k = m_vecTags.begin( ); k != m_vecTags.end( ); k++ )
					{
						if( (*k).first == pTorrents[j].strTag )
							if ( !(*k).second.empty( ) )
								pResponse->strContent += "<img src=\"" + (*k).second + "\" alt=\"[" + pTorrents[j].strTag + "]\" title=\"" + pTorrents[j].strTag + "\" name=\"" + pTorrents[j].strTag + "\">";
							else
								pResponse->strContent += UTIL_RemoveHTML( pTorrents[j].strTag );
					}
				}
				pResponse->strContent += "</td>\n" ;

				// display the name and stats link
				pResponse->strContent += "<td class=\"name\"><a title=\"" + gmapLANG_CFG["name"] + ": " + UTIL_RemoveHTML( pTorrents[j].strName )+ "\" href=\"/stats.html?info_hash=" + UTIL_HashToString( pTorrents[j].strInfoHash ) + "\">" + pTorrents[j].strName + "</a></td>\n";

				// display the added date
				pResponse->strContent += "<td class=\"date\">" + pTorrents[j].strAdded + "</td>\n";

				// display user's torrent administration if any
				if( m_bDeleteOwnTorrents )
					pResponse->strContent += "<td class=\"admin\">[<a title=\"" + gmapLANG_CFG["delete"] + ": " + UTIL_HashToString( pTorrents[j].strInfoHash ) + "\" href=\"/login.html?del=" + UTIL_HashToString( pTorrents[j].strInfoHash ) + "\">" + gmapLANG_CFG["delete"] + "</a>]</td>\n";
				else
					pResponse->strContent += "<td class=\"admin\">" + gmapLANG_CFG["na"] + "</td>\n";

				// end row
				pResponse->strContent += "</tr>\n";

				// increment row counter for row colour
				iAdded++;
			}

			// free the memory
			delete [] pTorrents;

			// close the table if created
			if( bFound )
			{
				pResponse->strContent += "</table>\n";
				pResponse->strContent += "</div>\n\n";
			}

			// Bottom search
			if( ( !strSearch.empty( ) || iFound ) && m_pAllowed && m_bSearch )
			{
				pResponse->strContent += "<form class=\"search_login\" name=\"bottomsearch\" method=\"get\" action=\"/login.html\">\n";

				if( !strSort.empty( ) )
					pResponse->strContent += "<p><input name=\"sort\" type=hidden value=\"" + strSort + "\"></p>\n";

				pResponse->strContent += "<p><label for=\"bottomtorrentsearch\">" + gmapLANG_CFG["torrent_search"] + "</label> <input name=\"search\" id=\"bottomtorrentsearch\" alt=\"[" + gmapLANG_CFG["torrent_search"] + "]\" type=text size=40> <a title=\"" + gmapLANG_CFG["clear_filter_search"] + "\" href=\"/index.html\">" + gmapLANG_CFG["clear_filter_search"] + "</a></p>\n";
				pResponse->strContent += "</form>\n\n";
			}
		}
		// ----------------------------------- End of Modification
	}

	// Modified by =Xotic=	
	// Output common HTML ending
	HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_LOGIN ) );
	// ---------------------- End of Modification
}
