/***
*
* BNBT Beta 8.0 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 Trevor Hogan
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

// =Xotic= Modified Source File

#include "bnbt.h"
#include "atom.h"
#include "bencode.h"
#include "client.h"
#include "tracker.h"
#include "util.h"

// Added by =Xotic=
#include "xconfig_lang.h"
// --------------------- End of Addition

void CTracker :: serverResponseUploadGET( struct request_t *pRequest, struct response_t *pResponse )
{
	struct bnbttv btv = UTIL_CurrentTime( );

	if( pRequest->user.iAccess & ACCESS_UPLOAD )
	{
		HTML_Common_Begin(  pRequest, pResponse, gmapLANG_CFG["upload_page"], string( CSS_UPLOAD ), string( ), NOT_INDEX, CODE_200 );
	
		if( m_strUploadDir.empty( ) )
		{
			// Modified by =Xotic=  
			// This tracker does not allow file uploads.
			pResponse->strContent += "<p class=\"denied\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_nofile_uploads"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_index"] + "\" href=\"/index.html\">" ).c_str( ), "</a>" ) + "</p>\n";

			// Output common HTML ending
			HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_UPLOAD ) );
			// ---------------------- End of Modification

			return;
		}
		else if( m_iMaxTorrents > 0 )
		{
			if( m_pAllowed && m_pAllowed->getValuePtr( )->size( ) >= (unsigned int)m_iMaxTorrents )
			{
				// Modified by =Xotic=  
				// This tracker has reached its torrent limit.
				pResponse->strContent += "<p class=\"denied\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_torrent_limit"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_index"] + "\" href=\"/index.html\">" ).c_str( ), "</a>" ) + "</p>\n";

				// Output common HTML ending
				HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_UPLOAD ) );
				// ---------------------- End of Modification

				return;
			}
		}

		pResponse->strContent += "<div class=\"torrent_upload\">\n";
		pResponse->strContent += "<table class=\"torrent_upload\">\n";
		pResponse->strContent += "<tr>\n";
		pResponse->strContent += "<td>\n";
		pResponse->strContent += "<form name=\"torrentupload\" method=\"post\" action=\"/upload.html\" enctype=\"multipart/form-data\">\n";
		// The Trinity Edition - Modification Begins
		// The following changes have been made:
		// 1. Field Descriptions now appear above the input fields
		// 2. Information regarding Optional input has been moved to a list below
		// 3. Modified the Tag field descriptor to read "Tag/Category"
		// 4. When FORCE_ANNOUNCE_URL is null/empty, the tracker's Announce URL will be displayed
		//    a CSS class "announce" is also set for this string, which can be used to HIDE the 
		//    "Announce URL" by using the following CSS command:
		//    .announce{display:none}
		// 5. Added a CANCEL button after the UPLOAD button.
		pResponse->strContent += "<p class=\"torrent_upload\"><label for=\"uploadfile\">" + gmapLANG_CFG["upload_file"] + "</label><br>\n";
		pResponse->strContent += "<input name=\"torrent\" id=\"uploadfile\" alt=\"[" + gmapLANG_CFG["torrent"] + "]\" type=file size=50><br></p>\n";        
		pResponse->strContent += "<p class=\"torrent_upload\"><label for=\"uploadname\">" + gmapLANG_CFG["upload_name"] + "</label><br>\n";
		pResponse->strContent += "<input name=\"name\" id=\"uploadname\" alt=\"[" + gmapLANG_CFG["name"] + "]\" type=text size=64 maxlength=" + CAtomInt( MAX_FILENAME_LEN ).toString( ) + "><br></p>\n";             

		if( m_bAllowInfoLink ) 
		{
			pResponse->strContent += "<p class=\"torrent_upload\"><label for=\"uploadinfolink\">" + gmapLANG_CFG["upload_infolink"] + "</label><br>\n";
			pResponse->strContent += "<input name=\"infolink\" id=\"uploadinfolink\" alt=\"[" + gmapLANG_CFG["link"] + "]\" type=text size=64 maxlength=" + CAtomInt( MAX_INFO_LINK_LEN ).toString( ) + " value=\"\"><br></p>\n";
		}

		if( m_vecTags.size( ) > 0 )
			pResponse->strContent += "<p class=\"torrent_upload_category\"><label for=\"uploadtag\">" + gmapLANG_CFG["upload_tag"] + "</label> <select id=\"uploadtag\" name=\"tag\">\n";

		if( m_vecTags.size( ) > 0 )
			for( vector< pair< string, string > >   :: iterator i = m_vecTags.begin( ); i != m_vecTags.end( ); i++ )
				pResponse->strContent += "<option>" + (*i).first + "\n";

		if( m_vecTags.size( ) > 0 )
			pResponse->strContent += "</select></p>\n";

		pResponse->strContent += "<ul>\n";
		pResponse->strContent += "<li class=\"torrent_upload\">" + gmapLANG_CFG["upload_note_1"] + "</li>\n";

		// Names must be less than %s characters long
		pResponse->strContent += "<li class=\"torrent_upload\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_note_2"].c_str( ), CAtomInt( MAX_FILENAME_LEN ).toString( ).c_str( ) ) + "</li>\n";
		
		pResponse->strContent += "<li class=\"torrent_upload\">" + gmapLANG_CFG["no_html"] + "</li>\n";
		pResponse->strContent += "<li class=\"torrent_upload\">" + gmapLANG_CFG["upload_note_3"] + "</li>\n";
		pResponse->strContent += "<li class=\"torrent_upload\"><strong>" + gmapLANG_CFG["upload_note_4"] + ":</strong> " + UTIL_BytesToString( giMaxRecvSize ) + "</li>\n";

		if( !m_strForceAnnounceURL.empty( ) )
			pResponse->strContent += "<li class=\"announce\"><strong>" + gmapLANG_CFG["forced_announce_url"] + ":</strong> " + UTIL_RemoveHTML( m_strForceAnnounceURL ) + "</li>\n";
		else
		{
			pResponse->strContent += "<li class=\"announce\"><strong>" + gmapLANG_CFG["announce_url"] + ":</strong> http://";
			pResponse->strContent += "<script type=\"text/javascript\">document.write( parent.location.host );</script>";
			pResponse->strContent += "/announce</li>\n";
		}

		pResponse->strContent += "</ul>\n";

		pResponse->strContent += "<div>\n";
		Button_Submit( "submit_upload", string( gmapLANG_CFG["upload"] ), pResponse );
		Button_Back( "cancel_upload", string( gmapLANG_CFG["cancel"] ), pResponse );
		pResponse->strContent += "\n</div>\n";
		// ------------------------------------------------- End of Modification
		pResponse->strContent += "</form>\n";
		pResponse->strContent += "</td>\n";
		pResponse->strContent += "</tr>\n";
		pResponse->strContent += "</table>\n";
		pResponse->strContent += "</div>\n";
	}
	else
		HTML_Common_Begin(  pRequest, pResponse, gmapLANG_CFG["upload_page"], string( CSS_UPLOAD ), string( ), NOT_INDEX, CODE_401 );

	// Modified by =Xotic=  
	// Output common HTML ending
	HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_UPLOAD ) );
	// ---------------------- End of Modification
}

void CTracker :: serverResponseUploadPOST( struct request_t *pRequest, struct response_t *pResponse, CAtomList *pPost )
{
	struct bnbttv btv = UTIL_CurrentTime( );

	if( m_strUploadDir.empty( ) || !( pRequest->user.iAccess & ACCESS_UPLOAD ) )
	{
		pResponse->strCode = "403 " + gmapLANG_CFG["server_response_403"];

		return;
	}

	if( m_iMaxTorrents > 0 )
	{
		if( m_pAllowed && m_pAllowed->getValuePtr( )->size( ) >= (unsigned int)m_iMaxTorrents )
		{
			pResponse->strCode = "403 " + gmapLANG_CFG["server_response_403"];

			return;
		}
	}

	string strFile;
	string strTorrent;
	string strTag;
	string strPostedName;
	string strPostedInfoLink;

	if( pPost )
	{
		vector<CAtom *> vecSegs = pPost->getValue( );

		for( unsigned long i = 0; i < vecSegs.size( ); i++ )
		{
			if( vecSegs[i]->isDicti( ) )
			{
				CAtomDicti *pSeg = (CAtomDicti *)vecSegs[i];

				CAtom *pDisp = pSeg->getItem( "disposition" );
				CAtom *pDat = pSeg->getItem( "data" );

				if( pDisp && pDisp->isDicti( ) && pDat )
				{
					CAtom *pName = ( (CAtomDicti *)pDisp )->getItem( "name" );

					if( pName )
					{
						string strName = pName->toString( );

						if( strName == "torrent" )
						{
							CAtom *pFile = ( (CAtomDicti *)pDisp )->getItem( "filename" );

							if( pFile )
							{
								// the path is local to the peer

								strFile = UTIL_StripPath( pFile->toString( ) );

								strTorrent = pDat->toString( );
							}
							else
							{
								pResponse->strCode = "400 " + gmapLANG_CFG["server_response_400"];

								return;
							}
						}
						else if( strName == "tag" )
							strTag = pDat->toString( );
						else if( strName == "name" )
							strPostedName = pDat->toString( ).substr( 0, MAX_FILENAME_LEN );
						else if( strName == "infolink" )
							strPostedInfoLink = pDat->toString( ).substr( 0, MAX_INFO_LINK_LEN );
					}
					else
					{
						pResponse->strCode = "400 " + gmapLANG_CFG["server_response_400"];

						return;
					}
				}
			}
		}
	}
	else
	{
		pResponse->strCode = "400 " + gmapLANG_CFG["server_response_400"];

		return;
	}

	// Modified by =Xotic=
	HTML_Common_Begin(  pRequest, pResponse, gmapLANG_CFG["upload_page"], string( CSS_UPLOAD ), string( ), NOT_INDEX, CODE_200 );
	// --------------------- End of Modification

	string strPath = m_strUploadDir + strFile;

	string :: size_type iExt = strFile.rfind( "." );

	string strExt;

	if( iExt != string :: npos )
		strExt = strFile.substr( iExt );

	if( strTorrent.empty( ) )
	{
		// Modified by =Xotic=
		pResponse->strContent += "<p class=\"failed\">" + gmapLANG_CFG["failed"] + "</p>\n";
		
		// The uploaded file is corrupt or invalid.
		pResponse->strContent += "<p class=\"body_upload\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_file_corrupt"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_upload"] + "\" href=\"/upload.html\">" ).c_str( ), "</a>" ) + "</p>\n";
		// ---------------------- End of Modification
	}
	else if( strExt != ".torrent" )
	{
		// Modified by =Xotic=
		pResponse->strContent += "<p class=\"failed\">" + gmapLANG_CFG["failed"] + "</p>\n";

		// The uploaded file is not a .torrent file.
		pResponse->strContent += "<p class=\"body_upload\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_not_torrent"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_upload"] + "\" href=\"/upload.html\">" ).c_str( ), "</a>" ) + "</p>\n";
		// ---------------------- End of Modification
	}
	else if( !checkTag( strTag ) )
	{
		// Modified by =Xotic=
		pResponse->strContent += "<p class=\"failed\">" + gmapLANG_CFG["failed"] + "</p>\n";
			
		// The file tag is invalid.
		pResponse->strContent += "<p class=\"body_upload\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_filetag_invalid"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_upload"] + "\" href=\"/upload.html\">" ).c_str( ), "</a>" ) + "</p>\n";
		// ---------------------- End of Modification
	}
	else if( UTIL_CheckFile( strPath.c_str( ) ) )
	{
		// Modified by =Xotic=
		pResponse->strContent += "<p class=\"failed\">" + gmapLANG_CFG["failed"] + "</p>\n";
			
		//The uploaded file already exists.
		pResponse->strContent += "<p class=\"body_upload\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_file_exists"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_upload"] + "\" href=\"/upload.html\">" ).c_str( ), "</a>" ) + "</p>\n";
		// ---------------------- End of Modification
	}
	else
	{
		CAtom *pTorrent = Decode( strTorrent );

		if( pTorrent && pTorrent->isDicti( ) )
		{
			string strInfoHash = UTIL_InfoHash( pTorrent );

			if( !strInfoHash.empty( ) )
			{
				if( m_pDFile->getItem( strInfoHash ) )
				{
					// Modified by =Xotic
					pResponse->strContent += "<p class=\"failed\">" + gmapLANG_CFG["failed"] + "</p>\n";
			
					// A file with the uploaded file's info hash already exists.
					pResponse->strContent += "<p class=\"body_upload\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_hash_exists"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_upload"] + "\" href=\"/upload.html\">" ).c_str( ), "</a>" ) + "</p>\n";
					// ---------------------- End of Modification
				}
				else
				{
					if( !m_strForceAnnounceURL.empty( ) )
						( (CAtomDicti *)pTorrent )->setItem( "announce", new CAtomString( m_strForceAnnounceURL ) );

					UTIL_MakeFile( strPath.c_str( ), Encode( pTorrent ) );

					addTag( strInfoHash, strTag, strPostedName, pRequest->user.strLogin, strPostedInfoLink, string( "false" ) );

					pResponse->strContent += "<p class=\"sucess\">" + gmapLANG_CFG["successful"] + "</p>\n";

					// Added by =Xotic=
					// The Trinity Edition - Addition Begins
					// The following adds an RTT link and a link to Upload Another Torrent
					// when a user has just SUCCESSFULLY UPLOADED A TORRENT - regardless of parsing method used
					pResponse->strContent += "<p class=\"return\"><a title=\"" + gmapLANG_CFG["navbar_upload"] + "\" href=\"/upload.html\">" + gmapLANG_CFG["upload_another"] + "</a><span class=\"pipe\"|</span><a title=\"" + gmapLANG_CFG["navbar_index"] + "\" href=\"/index.html\">" + gmapLANG_CFG["return_to_tracker"] + "</a></p>\n\n";
					// ------------------------------------------------- End of Addition                    

					// The Trinity Edition - Modification Begins
					// The following removes the multiple RTT links that appear based on parsing method used
					if( m_bParseOnUpload )
					{
						if( m_pAllowed )
							parseTorrent( strPath.c_str( ) );

						pResponse->strContent += "<p class=\"body_upload\">" + gmapLANG_CFG["upload_ready_now"] + "</p>\n";
					}
					else
					{
						// The uploaded file will be ready in
						if( m_iParseAllowedInterval == 1 )
							pResponse->strContent += "<p class=\"body_upload\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_ready_1min"].c_str( ), CAtomInt( m_iParseAllowedInterval ).toString( ).c_str( ) ) + "</p>\n";
						else
							pResponse->strContent += "<p class=\"body_upload\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_ready_xmins"].c_str( ), CAtomInt(  m_iParseAllowedInterval ).toString( ).c_str( ) ) + "</p>\n";
					}
					// ------------------------------------------------- End of Modification

					// Added by =Xotic=
					// The Trinity Edition - Addition Begins
					// The following displays SEEDING INSTRUCTIONS after a user successfully uploads a torrent
					pResponse->strContent += "<div class=\"seeding_instructions\">\n";
					pResponse->strContent += "<table class=\"seeding_instructions\">\n";
					pResponse->strContent += "<tr><td>\n";
					pResponse->strContent += "<p class=\"seeding_instructions_head\">" + gmapLANG_CFG["upload_seeding_instructions"] + "</p>\n";
					pResponse->strContent += "<p class=\"seeding_instructions_lead\">" + gmapLANG_CFG["upload_begin_seeding"] + "</p>\n";
					pResponse->strContent += "<ol>\n";
					pResponse->strContent += "<li class=\"seeding_instructions\">" + gmapLANG_CFG["upload_inst_1"] + "&dagger;</li>\n";
					pResponse->strContent += "<li class=\"seeding_instructions\">" + gmapLANG_CFG["upload_inst_2"] + "</li>\n";
					pResponse->strContent += "<li class=\"seeding_instructions\">" + gmapLANG_CFG["upload_inst_3"] + "</li>\n";
					pResponse->strContent += "<li class=\"seeding_instructions\">" + gmapLANG_CFG["upload_inst_4"] + "</li>\n";
					pResponse->strContent += "</ol>\n";
					pResponse->strContent += "<p class=\"seeding_instructions\">" + gmapLANG_CFG["upload_inst_verify"] + "<br>\n";
			
					// Return to the Tracker and examine the SDs (Seeders) column.
					pResponse->strContent += UTIL_Xsprintf( gmapLANG_CFG["upload_inst_return"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_index"] + "\"href=\"/index.html\">" ).c_str( ), "</a>" ) + "</p>\n";

					pResponse->strContent += "</td></tr></table>\n";
					
					// Instructions note
					pResponse->strContent += "<table class=\"seeding_instructions_note\"><tr><td style=\"font-weight:normal; border:1px solid black; padding:10px\">\n";
					pResponse->strContent += "<p class=\"seeding_instructions\">&dagger; " + gmapLANG_CFG["upload_note_1"] + "<br>\n";
					pResponse->strContent += gmapLANG_CFG["upload_note_2"] + "<br>\n";
					pResponse->strContent += gmapLANG_CFG["upload_note_3"] + "</p>\n";
					pResponse->strContent += "</td></tr></table>\n";
					pResponse->strContent += "</div>\n";
					// ------------------------------------------------- End of Addition

					UTIL_LogPrint( string( gmapLANG_CFG["user_uploaded_torrent"] + "\n" ).c_str( ), inet_ntoa( pRequest->sin.sin_addr ), strFile.c_str( ) );
				}
			}
			else
			{
				// Modified by =Xotic=
				pResponse->strContent += "<p class=\"failed\">" + gmapLANG_CFG["failed"] + "</p>\n";
					
				// The uploaded file is corrupt or invalid.
				pResponse->strContent += "<p class=\"body_upload\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_file_corrupt"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_upload"] + "\" href=\"/upload.html\">" ).c_str( ), "</a>" ) + "</p>\n";
				// ---------------------------------- End of Modification
			}
		}
		else
		{
			// Modified by =Xotic=  
			pResponse->strContent += "<p class=\"failed\">" + gmapLANG_CFG["failed"] + "</p>\n";

			// The uploaded file is corrupt or invalid.
			pResponse->strContent += "<p class=\"body_upload\">" + UTIL_Xsprintf( gmapLANG_CFG["upload_file_corrupt"].c_str( ), string( "<a title=\"" + gmapLANG_CFG["navbar_upload"] + "\" href=\"/upload.html\">" ).c_str( ), "</a>" ) + "</p>\n";
			// ---------------------------------- End of Modification
		}

		if( pTorrent )
			delete pTorrent;
	}

	// Modified by =Xotic=  
	// Output common HTML ending
	HTML_Common_End( pRequest, pResponse, btv, NOT_INDEX, string( CSS_UPLOAD ) );
	// ---------------------- End of Modification
}
