/***
*
* XBNBT Beta 81b.3.4 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 =Xotic=
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

#include "bnbt.h"
#include "atom.h"
#include "bencode.h"
#include "config.h"
#include "tracker.h"
#include "util.h"

// XBNBT Language support
#include "xconfig_lang.h"   

// Initialise XBNBT CBTT feature variables
void CTracker :: initCBTTVars( )
{
	//DWMod Ban Code
	i_intBanMode = CFG_GetInt( "cbtt_ban_mode", 0 );
	i_intPeerSpoofRestrict = CFG_GetInt( "cbtt_restricted_peer_spoofing", 0 );
	m_strClientBanFile = CFG_GetString( "cbtt_ban_file", "clientbans.bnbt" );

	// CBTT IP banning Mode
	m_iIPBanMode = CFG_GetInt( "cbtt_ip_ban_mode", 0 );
	m_strIPBanFile = CFG_GetString( "cbtt_ipban_file", "bans.bnbt" );

	// misc CBTT mod functions
	m_iDontCompressTorrent = CFG_GetInt( "cbtt_dont_compress_torrents", 0 );
	m_iRestrictOverflow = CFG_GetInt( "cbtt_restrict_overflow", 0 );
	strOverFlowLimit = CFG_GetString( "cbtt_restrict_overflow_limit", "1099511627776" );
	m_iBlockNATedIP = CFG_GetInt( "cbtt_block_private_ip", 0 );
	m_iTorrentTraderCompatibility = CFG_GetInt( "mysql_cbtt_ttrader_support", 0 );
	m_iBlacklistP2PPorts = CFG_GetInt( "cbtt_blacklist_common_p2p_ports",0);
	
	// adopted official tracker config value
	m_iLocalOnly = CFG_GetInt( "only_local_override_ip", 0 );  
}

void CTracker :: initCBTTParse( void ) 
{ 
	// DWKMod Ban Stuff
	m_pClientBannedList = new CAtomList();
	parseClientBanList();
	UTIL_LogPrint( string( gmapLANG_CFG["cbtt_client_parse"] + "\n" ).c_str( ) );

	m_pIPBannedList = new CAtomList();
	parseIPBanList();
	UTIL_LogPrint( string( gmapLANG_CFG["cbtt_ip_parse"] + "\n" ).c_str( ) );
}

// DWMOD ban stuff 
void CTracker :: parseClientBanList( void ) 
{ 
	m_pClientBannedList->clear(); 
	string text = UTIL_ReadFile( m_strClientBanFile.c_str( ) ); 
	string separators = "\n\r"; 
	int n = ( int )text.length(); 
	int start, stop; 

	// loop through the ban file and add 
	start = ( int )text.find_first_not_of(separators); 
	while ((start >= 0) && (start < n)) { 
		stop = ( int )text.find_first_of(separators, start); 
		if ((stop < 0) || (stop > n)) 
			stop = n; 

		m_pClientBannedList->addItem(new CAtomString(text.substr(start, stop - start))); 
		start = ( int )text.find_first_not_of(separators, stop + 1); 
	} 
} 

void CTracker :: parseIPBanList( void ) 
{ 
	m_pIPBannedList->clear(); 
	string text = UTIL_ReadFile( m_strIPBanFile.c_str( ) ); 
	string separators = "\n\r "; 
	int n = ( int )text.length(); 
	int start, stop; 

	// loop through the ban file and add shit 
	start = ( int )text.find_first_not_of(separators); 
	while ((start >= 0) && (start < n)) { 
		stop = ( int )text.find_first_of(separators, start); 
		if ((stop < 0) || (stop > n)) 
			stop = n; 

		m_pIPBannedList->addItem(new CAtomString(text.substr(start, stop - start))); 
		start = ( int )text.find_first_not_of(separators, stop + 1); 
	} 
} 

// Client Banlist check
int CTracker :: isIPBanList( string strPeerIP )
{
	vector<CAtom *>list = ( (CAtomList *)m_pIPBannedList )->getValue( ); 
	for( unsigned long i = 0; i < list.size( ); i++ )
	{ 
		CAtomString *str = (CAtomString *)list[i]; 
		string currentline = str->getValue();
		if ( currentline == strPeerIP )
			return 1;
	}
	return 0;
}

int CTracker :: isClientBanList( string strPeerID, bool isUserAgent ) 
{
	string tempPeerID;
	if( isUserAgent )
		tempPeerID = strPeerID;
	else
		tempPeerID = UTIL_StringToEscaped(strPeerID);
	vector<CAtom *>list = ( (CAtomList *)m_pClientBannedList )->getValue( ); 
	for( unsigned long i = 0; i < list.size( ); i++ ) 
	{ 
		CAtomString *str = (CAtomString *)list[i]; 
		string currentline = str->getValue();
		if ( currentline == tempPeerID.substr(0,currentline.end()-currentline.begin()) && tempPeerID != "" ) {
			return 1;
		} 
		else if ( currentline == tempPeerID.c_str() ) {
			return 1;
		} 
	}
	return 0; 
} 

// CBTT responses
void CTracker :: serverResponseIcon( struct response_t *pResponse )
{
	pResponse->strCode = "200 " + gmapLANG_CFG["server_response_200"];

	pResponse->mapHeaders.insert( pair<string, string>( "Content-Type", gmapMime[".ico"] ) );

	pResponse->strContent = favicon.strFile;
}
