/***
*
* XBNBT Beta 81b.3.4 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 =Xotic=
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

// Modified by =Xotic=, from bnbt_mysql.cpp

#if defined XBNBT_MYSQL || defined BNBT_MYSQL

#include "bnbt.h"
#include "xmysqlusers.h"
#include "util.h"
#include "atom.h"
#include "config.h"
#include "xconfig_lang.h"

MYSQL *gpMySQLUsers;
string gstrMySQLUsersHost;
string gstrMySQLUsersDatabase;
string gstrMySQLUsersUser;
string gstrMySQLUsersPassword;
string gstrMySQLUsersTable;
string gstrMySQLUsersAdmin;
int giMySQLUsersPort;
int giMySQLUsersOverrideUsers;
int giMySQLUsersMemberAccess;
string gstrMySQLUsersName;

void UTIL_InitMySQLUsersVars( )
{
	gstrMySQLUsersHost = CFG_GetString( "xbnbt_mysqlusers_host", string( ) );
	gstrMySQLUsersDatabase = CFG_GetString( "xbnbt_mysqlusers_database", string( ) );
	gstrMySQLUsersUser = CFG_GetString( "xbnbt_mysqlusers_user", string( ) );
	gstrMySQLUsersPassword = CFG_GetString( "xbnbt_mysqlusers_password", string( ) );
	giMySQLUsersPort = CFG_GetInt( "xbnbt_mysqlusers_port", 3306 );
	giMySQLUsersOverrideUsers = CFG_GetInt( "xbnbt_mysqlusers_override_users", 0 );
    giMySQLUsersMemberAccess = CFG_GetInt( "bnbt_member_access", ACCESS_VIEW | ACCESS_DL | ACCESS_COMMENTS | ACCESS_UPLOAD | ACCESS_SIGNUP );
	gstrMySQLUsersTable = CFG_GetString( "xbnbt_mysqlusers_table", string( ) );
	gstrMySQLUsersAdmin = CFG_GetString( "xbnbt_mysqlusers_forums_admin", string( ) );
	gstrMySQLUsersName = CFG_GetString( "xbnbt_mysqlusers_table_name", string( ) );
}

bool UTIL_StartMySQLUsers( )
{
	bool bResult = false;
	int iCount = 0;

	MILLISLEEP( 100 );
	while( !bResult && ( iCount < 5 ) )
	{
		bResult = UTIL_OpenMySQLUsers( );
		iCount++;
		MILLISLEEP( 1000 );
	}

	if( bResult )
	{
		if( UTIL_TestMySQLUsersColumn( ) )
			bResult = UTIL_SetMySQLUsersMemberDefaultAccess( );
		else
			bResult = UTIL_CreateMySQLUsersColumn( );

		if( bResult )
			bResult = UTIL_SetMySQLUsersMemberAccess( gstrMySQLUsersAdmin, 127 );
	}

	return bResult;
}

string UTIL_StringToMySQLUsers( const string &strString )
{
	char *szMySQLUsers = new char[strString.size( ) * 2 + 1];

	if( gpMySQLUsers )
		mysql_real_escape_string( gpMySQLUsers, szMySQLUsers, strString.c_str( ), ( unsigned long )strString.size( ) );

	string strMySQLUsers = szMySQLUsers;

	delete [] szMySQLUsers;

	return strMySQLUsers;
}

bool UTIL_OpenMySQLUsers(void)
{
	// Start MySQL Users mysql
	gpMySQLUsers = mysql_init( NULL );
	if( !gpMySQLUsers )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["init_mysql_users"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		return false;
	}
	
	if( !( mysql_real_connect( gpMySQLUsers, gstrMySQLUsersHost.c_str( ), gstrMySQLUsersUser.c_str( ), gstrMySQLUsersPassword.c_str( ), gstrMySQLUsersDatabase.c_str( ), giMySQLUsersPort, NULL, 0 ) ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["connect_mysql_users"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["success_mysql_users"] + "\n" ).c_str( ) );
	return true;
}

bool UTIL_TestMySQLUsersColumn(void)
{
	string strQuery = "SELECT xbnbt_perm FROM " + gstrMySQLUsersTable + " LIMIT 1";

	mysql_real_query( gpMySQLUsers, strQuery.c_str( ), ( unsigned long )strQuery.size( ) );

	mysql_store_result( gpMySQLUsers );

	if( mysql_errno( gpMySQLUsers ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["no_mysql_users_column"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["found_mysql_users_column"] + "\n" ).c_str( ) );
	return true;
}

bool UTIL_SetMySQLUsersMemberDefaultAccess(void)
{
	string strQuery = "ALTER TABLE " + gstrMySQLUsersTable + " ALTER COLUMN xbnbt_perm SET DEFAULT \'" + CAtomInt( giMySQLUsersMemberAccess ).toString( ) + "\'";
	
	mysql_real_query( gpMySQLUsers, strQuery.c_str( ), ( unsigned long )strQuery.size( ) );

	mysql_store_result( gpMySQLUsers );
	
	if( mysql_errno( gpMySQLUsers ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["not_set_default_access"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["set_default_access"] + "\n" ).c_str( ) );
	return true;
}

bool UTIL_SetMySQLUsersMemberAccess( string strMySQLUsersMember, int iMySQLUsersAccess )
{
	string strQuery = "UPDATE " + gstrMySQLUsersTable + " SET xbnbt_perm=\'" + CAtomInt( iMySQLUsersAccess ).toString( ) + "\' WHERE " + gstrMySQLUsersName + "=\'" + strMySQLUsersMember + "\'";

	mysql_real_query( gpMySQLUsers, strQuery.c_str( ), ( unsigned long )strQuery.size( ) );

	mysql_store_result( gpMySQLUsers );
	
	if( mysql_errno( gpMySQLUsers ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["not_set_member_access"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["set_member_access"] + "\n" ).c_str( ) );
	return true;
}

bool UTIL_CreateMySQLUsersColumn(void)
{
	string strQuery = "ALTER TABLE " + gstrMySQLUsersTable + " ADD COLUMN xbnbt_perm SMALLINT DEFAULT \'" + CAtomInt( giMySQLUsersMemberAccess ).toString( ) + "\'" ;

	mysql_real_query( gpMySQLUsers, strQuery.c_str( ), ( unsigned long )strQuery.size( ) );

	mysql_store_result( gpMySQLUsers );

	if( mysql_errno( gpMySQLUsers ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["not_created_access_column"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["created_access_column"] + "\n" ).c_str( ) );
	return true;
}

bool UTIL_CloseMySQLUsers(void)
{
	mysql_close( gpMySQLUsers );

	if( mysql_errno( gpMySQLUsers ) )
	{
		if( gbDebug )
			UTIL_LogPrint( string( gmapLANG_CFG["not_closed_mysql_users"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		return false;
	}

	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["closed_mysql_users"] + "\n" ).c_str( ) );
	return true;
}

CMySQLUsersQuery :: CMySQLUsersQuery( string strQuery )
{
	if( gbDebug )
		UTIL_LogPrint( string( gmapLANG_CFG["mysql_users_query"] + "\n" ).c_str( ), strQuery.c_str( ) );

	if( !mysql_ping( gpMySQLUsers ) )
	{
		mysql_real_query( gpMySQLUsers, strQuery.c_str( ), ( unsigned long )strQuery.size( ) );

		m_pRes = mysql_store_result( gpMySQLUsers );

		if( mysql_errno( gpMySQLUsers ) )
		{
			m_pRes = NULL;

			if( gbDebug )
				UTIL_LogPrint( string( gmapLANG_CFG["mysql_users_query_error"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		}
	}
	else
	{
		if( mysql_errno( gpMySQLUsers ) )
		{
			m_pRes = NULL;
			
			if( gbDebug )
				UTIL_LogPrint( string( gmapLANG_CFG["ping_error"] + "\n" ).c_str( ), mysql_error( gpMySQLUsers ) );
		}
	}
}

CMySQLUsersQuery :: ~CMySQLUsersQuery( )
{
	if( m_pRes )
		mysql_free_result( m_pRes );
}

vector<string> CMySQLUsersQuery :: nextRow( )
{
	vector<string> vecReturn;

	if( m_pRes )
	{
		MYSQL_ROW row;

		unsigned int num_fields = mysql_num_fields( m_pRes );
		unsigned int i;

		vecReturn.reserve( num_fields );

		row = mysql_fetch_row( m_pRes );
		if( row )
		{
			unsigned long *lengths = mysql_fetch_lengths( m_pRes );

			for( i = 0; i < num_fields; i++ )
				vecReturn.push_back( string( row[i], lengths[i] ) );
		}
	}

	return vecReturn;
}

#endif
