/***
*
* XBNBT Beta 81b.3.4 - A C++ BitTorrent Tracker
* Copyright (C) 2003-2004 =Xotic=
*
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
* This library is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this library; if not, write to the Free Software
* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*
***/

// Modified by =Xotic=, from util.h

#ifndef WIN32
 #include <sys/time.h>
 #include <stdarg.h>
#endif

#ifdef WIN32
 #include <io.h>
#endif

#include "bnbt.h"
#include "atom.h"
#include "tracker.h"
#include "xutil.h"

//addition by labarks
string UTIL_Date( )
{
	// format date

	time_t tNow = time( NULL );

	char pTime[256];
	memset( pTime, 0, sizeof( char ) * 256 );
	strftime( pTime, sizeof( char ) * 256, "%a, %d %b %Y %H:%M:%S", localtime( &tNow ) );

	// Modified by =Xotic=
#if defined( __APPLE__ ) || defined( __FREEBSD__ ) || defined( __OPENBSD__ ) || defined( __NETBSD__ )
	// ----------------- End of Modification
	long timezone = -( localtime( &tNow )->tm_gmtoff );
#endif

	string strDate = pTime;

	// timezone has the wrong sign, change it

	if( timezone > 0 )
		strDate += " -";
	else
		strDate += " +";

	if( abs( timezone / 3600 ) % 60 < 10 )
		strDate += "0";

	strDate += CAtomInt( abs( timezone / 3600 ) % 60 ).toString( );

	if( abs( timezone / 60 ) % 60 < 10 )
		strDate += "0";

	strDate += CAtomInt( abs( timezone / 60 ) % 60 ).toString( );

	return strDate;
}
// labarks end

// XBNBT
bool UTIL_CheckDir( const char *szFile )
{
	// check if directory exists
	if( (access( szFile, 0 ) ) != -1  )
		return true;
	else
		return false;
}

// XBNBT
string UTIL_Xsprintf( const char *format, ... )
{
	char pBufTemp[LANG_BUF_LEN];
	memset( pBufTemp, 0, sizeof( char ) * LANG_BUF_LEN );
	va_list args;
	va_start( args, format );
	//va_arg( args, char * );
	vsprintf( pBufTemp, format, args );
	va_end( args );
	return string( pBufTemp );
}

// From jTracker 1.25
string UTIL_64ToString(int64 v)
{
	char b[21];
#ifdef WIN32
	sprintf(b, "%I64d", v);
#else
	sprintf(b, "%lld", v);
#endif
	return b;
}

// hex encode 
string UTIL_HexEncode(int l, int v)
{
	string r;
	r.resize(l);
	while (l--)
	{
		r[l] = "0123456789abcdef"[v & 0xf];
		v >>= 4;
	}
	return r;
}

int UTIL_HexDecode(char v)
{
	if( v >= '0' && v <= '9' )
		return v - '0';
	if( v >= 'A' && v <= 'Z' )
		return v - 'A' + 10;
	if( v >= 'a' && v <= 'z' )
		return v - 'a' + 10;
	return -1;
};

string UTIL_HexToString( const string& sHex )
{
	string sString;
	sString.reserve( sHex.length() << 1 );
	for( DWORD i = 0; i < sHex.size( ); i++ )
	{
#if 0
		char pBuf[4];
		memset( pBuf, 0, sizeof( char ) * 4 );
		// this must be unsigned or some really strange errors appear (i.e. the value of i is reset to zero every loop)
		unsigned char c = sHex[i];
		sprintf( pBuf, "%02x", c );
		sString += pBuf;
#else
		sString +=UTIL_HexEncode( 2, sHex[i] );
#endif
	}
	return sString;
}

string UTIL_StringToHex(const string& sString)
{
	string sHex;
	sHex.resize(sString.length() >> 1);
#if 0
	for( DWORD i = 0; i < sString.size( ); i += 2 )
	{
		char pBuf[4];
		memset( pBuf, 0, sizeof( char ) * 4 );
		pBuf[0] = sString[i];
		pBuf[1] = sString[i + 1];
		unsigned int c;
		sscanf( pBuf, "%02x", &c );
		sHex += c;
	}
#else
	int a = 0;
	for( DWORD i = 0; i + 2 <= sString.length(); i += 2 )
	{
		a =  UTIL_HexDecode(sString[i]) << 4;
		a |= UTIL_HexDecode(sString[i + 1]);
		sHex[i >> 1] = (char) a;
	}
#endif
	return sHex;
}

// PeerID convserion utils
// some code taken from XBT
static string sPeerID2Client(const string& name, const string& peer_id, int i)
{
	int j;
	for (j = i; j < 7; j++)
	{
		if (!isalnum(peer_id[j]))
			break;
	}
	return name + peer_id.substr(i, j - i);
}

string UTIL_PeerID2String(const string& sPeerID, string& sClientType)
{
	sClientType = "";

	if( sPeerID.length() != 20 )
		return UTIL_HexToString( sPeerID );

	// check for known clients
	if( sPeerID[7] == '-' )
	{
		switch( sPeerID[0] )
		{
		case '-':	// azureus
			if( sPeerID[1] == 'A' && sPeerID[2] == 'Z' )
			{
				sClientType	= "Azureus ";
				sClientType += sPeerID[3];
				sClientType += '.';
				sClientType += sPeerID[4];
				sClientType += '.';
				sClientType += sPeerID[5];
				sClientType += '.';
				sClientType += sPeerID[6];
				return UTIL_HexToString(sPeerID.substr(8));
			}
			break;
		case 'T':	// BitTornado
			sClientType	= "BitTornado ";
			sClientType += sPeerID[1];
			sClientType += '.';
			sClientType += sPeerID[2];
			sClientType += '.';
			sClientType += sPeerID[3];
			return UTIL_HexToString(sPeerID.substr(8));
		case 'M':	// MainLine	
			sClientType	= sPeerID2Client("Mainline ", sPeerID, 1);
			return UTIL_HexToString(sPeerID.substr(8));
		case 'A':	// ABC	
			sClientType	= sPeerID2Client("ABC ", sPeerID, 1);
			return UTIL_HexToString(sPeerID.substr(8));
		case 'S':	// Shadows experimental
			sClientType	= sPeerID2Client("Shadow ", sPeerID, 1);
			return UTIL_HexToString(sPeerID.substr(8));
		case 'X':	// XBT
			if( sPeerID[1] == 'B' && sPeerID[2] == 'T' )
			{
				sClientType	= "XBT Client ";
				sClientType += sPeerID[3];
				sClientType += '.';
				sClientType += sPeerID[4];
				sClientType += '.';
				sClientType += sPeerID[5];
				return UTIL_HexToString(sPeerID.substr(8));
			}		// Torment
			else if( sPeerID[1] == '^'  )
			{
				sClientType	= sPeerID2Client("Torment ", sPeerID, 3);
				return UTIL_HexToString(sPeerID.substr(8));
			}
			break;
		}
	}

	// bit comet
	if( memcmp( sPeerID.c_str(), "exbc", 4) == 0 )
	{
		sClientType = "BitComet ";
		sClientType += CAtomInt(sPeerID[4]).toString( );
		sClientType += '.';
		sClientType += CAtomInt(((uint8)sPeerID[5])/10).toString( );
		sClientType += '.';
		sClientType += CAtomInt(((uint8)sPeerID[5])%10).toString( );
		return UTIL_HexToString(sPeerID.substr(8));
	}
	else
	{
		switch( sPeerID[0] )
		{
		case 'U':	// UPnP
			sClientType = "UPnP " + sPeerID.substr(1,3);
			return UTIL_HexToString(sPeerID.substr(4));
			break;
		case 'S':	// Shadlow 5.7 ?
			if (sPeerID[1] == 5 && sPeerID[2] == 7 && sPeerID[3] >= 0 && sPeerID[3] < 10)
			{
				sClientType = "Shadow 57" + UTIL_64ToString(sPeerID[3]);
				return UTIL_HexToString(sPeerID.substr(4));
			}
			break;
		case 0:
			{
				sClientType = "0";
				unsigned int i = ( unsigned int )sPeerID.find_first_not_of('\0');
				if( i != string::npos )
					return i < 20 ? UTIL_HexToString(sPeerID.substr(i)) : "0";
			}
			break;
		}
	}

	sClientType = "Unk: " + sPeerID.substr(8);
	return UTIL_HexToString(sPeerID);
}
